@@ -26,6 +26,7 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
@@ -38,7 +39,6 @@
 
 #include "bacon-message-connection.h"
 #include "sj-about.h"
-#include "sj-genres.h"
 #include "sj-metadata.h"
 #include "sj-metadata-musicbrainz.h"
 #include "sj-extractor.h"
@@ -46,6 +46,8 @@
 #include "sj-error.h"
 #include "sj-util.h"
 #include "sj-prefs.h"
+#include "sj-play.h"
+#include "sj-volume.h"
 
 gboolean on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
 
@@ -64,12 +66,13 @@ SjExtractor *extractor;
 GConfClient *gconf_client;
 
 GtkWidget *main_window;
-static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_combo;
-static GtkWidget *track_listview, *extract_button;
+static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry;
+static GtkWidget *track_listview, *extract_button, *play_button;
 static GtkWidget *status_bar;
-static GtkWidget *extract_menuitem, *select_all_menuitem, *deselect_all_menuitem;
+static GtkWidget *extract_menuitem, *play_menuitem, *select_all_menuitem, *deselect_all_menuitem;
 GtkListStore *track_store;
 static BaconMessageConnection *connection;
+GtkCellRenderer *toggle_renderer, *title_renderer, *artist_renderer;
 
 const char *base_uri, *path_pattern, *file_pattern;
 NautilusBurnDrive *drive = NULL;
@@ -82,13 +85,85 @@ static gint total_no_of_tracks;
 static gint no_of_tracks_selected;
 static AlbumDetails *current_album;
 
-int autostart = FALSE;
+gboolean autostart = FALSE, autoplay = FALSE;
 
 #define DEFAULT_PARANOIA 4
 #define RAISE_WINDOW "raise-window"
 #define SOURCE_GLADE "../data/sound-juicer.glade"
 #define INSTALLED_GLADE DATADIR"/sound-juicer/sound-juicer.glade"
 
+static void
+add_stock_icon (GtkIconFactory *sj_icon_factory,
+                const gchar *stock_id,
+                GtkIconSize  size,
+                const gchar *file_name)
+{
+  GtkIconSource *source;
+  GtkIconSet    *set;
+  GdkPixbuf     *pixbuf;
+
+  source = gtk_icon_source_new ();
+
+  gtk_icon_source_set_size (source, size);
+  gtk_icon_source_set_size_wildcarded (source, FALSE);
+
+  pixbuf = gdk_pixbuf_new_from_file (file_name, NULL);
+
+  gtk_icon_source_set_pixbuf (source, pixbuf);
+
+  g_object_unref (pixbuf);
+
+  set = gtk_icon_set_new ();
+
+  gtk_icon_set_add_source (set, source);
+  gtk_icon_source_free (source);
+
+  gtk_icon_factory_add (sj_icon_factory, stock_id, set);
+
+  gtk_icon_set_unref (set);
+}
+
+void
+sj_stock_init (void)
+{
+  static gboolean initialized = FALSE;
+  static GtkIconFactory *sj_icon_factory = NULL;
+
+  static GtkStockItem sj_stock_items[] =
+  {
+    { SJ_STOCK_PLAYING, NULL, 0, 0, NULL },
+    { SJ_STOCK_RECORDING, NULL, 0, 0, NULL },
+    { SJ_STOCK_EXTRACT, "E_xtract", GDK_CONTROL_MASK, GDK_Return, NULL }
+  };
+
+  if (initialized)
+    return;
+
+  sj_icon_factory = gtk_icon_factory_new ();
+
+  add_stock_icon (sj_icon_factory, SJ_STOCK_PLAYING, GTK_ICON_SIZE_MENU, PKGDATADIR"/sj-play.png");
+  add_stock_icon (sj_icon_factory, SJ_STOCK_RECORDING, GTK_ICON_SIZE_MENU, PKGDATADIR"/sj-record.png");
+  gtk_icon_factory_add (sj_icon_factory, SJ_STOCK_EXTRACT, gtk_icon_factory_lookup_default (GTK_STOCK_CDROM));
+
+  gtk_icon_factory_add_default (sj_icon_factory);
+
+  gtk_stock_add_static (sj_stock_items, G_N_ELEMENTS (sj_stock_items));
+
+  initialized = TRUE;
+}
+
+void
+sj_main_set_title (const char* detail)
+{
+  if (detail == NULL) {
+    gtk_window_set_title (GTK_WINDOW (main_window), _("Sound Juicer"));
+  } else {
+    char *s = g_strdup_printf ("%s - %s", detail, _("Sound Juicer"));
+    gtk_window_set_title (GTK_WINDOW (main_window), s);
+    g_free (s);
+  }
+}
+
 static void error_on_start (GError *error)
 {
   GtkWidget *dialog;
@@ -127,6 +202,9 @@ void on_destroy_signal (GtkMenuItem *item, gpointer user_data)
  */
 void on_eject_activate (GtkMenuItem *item, gpointer user_data)
 {
+  /* first make sure we're not playing */
+  stop_playback ();
+
   nautilus_burn_drive_eject (drive);
 }
 
@@ -226,6 +304,29 @@ static void duration_cell_data_cb (GtkTreeViewColumn *tree_column,
   g_free (string);
 }
 
+static void number_cell_icon_data_cb (GtkTreeViewColumn *tree_column,
+				      GtkCellRenderer *cell,
+				      GtkTreeModel *tree_model,
+				      GtkTreeIter *iter,
+				      gpointer data)
+{
+  TrackState state;
+  gtk_tree_model_get (tree_model, iter, COLUMN_STATE, &state, -1);
+  switch (state) {
+  case STATE_IDLE:
+    g_object_set (G_OBJECT (cell), "stock-id", "", NULL);
+    break;
+  case STATE_PLAYING:
+    g_object_set (G_OBJECT (cell), "stock-id", SJ_STOCK_PLAYING, NULL);
+    break;
+  case STATE_EXTRACTING:
+    g_object_set (G_OBJECT (cell), "stock-id", SJ_STOCK_RECORDING, NULL);
+    break;
+  default:
+    g_warning("Unhandled track state %d\n", state);
+  }
+}
+
 /**
  * Utility function to update the UI for a given Album
  */
@@ -240,11 +341,13 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_list_store_clear (track_store);
     gtk_entry_set_text (GTK_ENTRY (title_entry), "");
     gtk_entry_set_text (GTK_ENTRY (artist_entry), "");
+    gtk_entry_set_text (GTK_ENTRY (genre_entry), "");
     gtk_label_set_text (GTK_LABEL (duration_label), "");
-    gtk_combo_box_set_active (GTK_COMBO_BOX (genre_combo), -1);
     gtk_widget_set_sensitive (title_entry, FALSE);
     gtk_widget_set_sensitive (artist_entry, FALSE);
-    gtk_widget_set_sensitive (genre_combo, FALSE);
+    gtk_widget_set_sensitive (genre_entry, FALSE);
+    gtk_widget_set_sensitive (play_button, FALSE);
+    gtk_widget_set_sensitive (play_menuitem, FALSE);
     gtk_widget_set_sensitive (extract_button, FALSE);
     gtk_widget_set_sensitive (extract_menuitem, FALSE);
     gtk_widget_set_sensitive (select_all_menuitem, FALSE);
@@ -259,10 +362,11 @@ static void update_ui_for_album (AlbumDetails *album)
     g_signal_handlers_unblock_by_func (title_entry, on_title_edit_changed, NULL);
     g_signal_handlers_unblock_by_func (artist_entry, on_artist_edit_changed, NULL);
 
-    gtk_combo_box_set_active (GTK_COMBO_BOX (genre_combo), -1);
     gtk_widget_set_sensitive (title_entry, TRUE);
     gtk_widget_set_sensitive (artist_entry, TRUE);
-    gtk_widget_set_sensitive (genre_combo, TRUE);
+    gtk_widget_set_sensitive (genre_entry, TRUE);
+    gtk_widget_set_sensitive (play_button, TRUE);
+    gtk_widget_set_sensitive (play_menuitem, TRUE);
     gtk_widget_set_sensitive (extract_button, TRUE);
     gtk_widget_set_sensitive (extract_menuitem, TRUE);
     gtk_widget_set_sensitive (select_all_menuitem, FALSE);
@@ -274,6 +378,7 @@ static void update_ui_for_album (AlbumDetails *album)
       album_duration += track->duration;
       gtk_list_store_append (track_store, &iter);
       gtk_list_store_set (track_store, &iter,
+                          COLUMN_STATE, STATE_IDLE,
                           COLUMN_EXTRACT, TRUE,
                           COLUMN_NUMBER, track->number,
                           COLUMN_TITLE, track->title,
@@ -283,7 +388,7 @@ static void update_ui_for_album (AlbumDetails *album)
                           -1);
      total_no_of_tracks++; 
     }
-no_of_tracks_selected=total_no_of_tracks;
+    no_of_tracks_selected=total_no_of_tracks;
 
     /* Some albums don't have track durations :( */
     if (album_duration) {
@@ -517,6 +622,11 @@ metadata_cb (SjMetadata *m, GList *albums, GError *error)
   
   if (autostart) {
     g_signal_emit_by_name (extract_button, "activate", NULL);
+    autostart = FALSE;
+  }
+  if (autoplay) {
+    g_signal_emit_by_name (play_button, "activate", NULL);
+    autoplay = FALSE;
   }
 }
 
@@ -866,20 +976,12 @@ void on_artist_edit_changed(GtkEditable *widget, gpointer user_data) {
   } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(track_store), &iter));
 }
 
-static gboolean genre_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {
-  TrackDetails *track;
-  gtk_tree_model_get (model, iter, COLUMN_DETAILS, &track, -1);
-  track->genre = GPOINTER_TO_INT(data);
-  return FALSE;
-}
-
-void on_genre_combo_changed(GtkComboBox *combo, gpointer user_data) {
-  GtkTreeIter iter;
-  if (gtk_combo_box_get_active_iter (combo, &iter)) {
-    int num;
-    gtk_tree_model_get (gtk_combo_box_get_model (combo), &iter, 0, &num, -1);
-    gtk_tree_model_foreach (GTK_TREE_MODEL (track_store), (GtkTreeModelForeachFunc)genre_foreach, GINT_TO_POINTER(num));
+void on_genre_edit_changed(GtkEditable *widget, gpointer user_data) {
+  g_return_if_fail (current_album != NULL);
+  if (current_album->genre) {
+    g_free (current_album->genre);
   }
+  current_album->genre = gtk_editable_get_chars (widget, 0, -1); /* get all the characters */
 }
 
 void on_contents_activate(GtkWidget *button, gpointer user_data) {
@@ -904,44 +1006,49 @@ void on_contents_activate(GtkWidget *button, gpointer user_data) {
   }
 }
 
-static int genre_cmp (gconstpointer a, gconstpointer b)
-{
-  GenreMap *p1, *p2;
-  p1 = (GenreMap *)a;
-  p2 = (GenreMap *)b;
-  return strcmp (sj_genre_name (p1->canonical), sj_genre_name (p2->canonical));
-}
 
-static GtkTreeModel* populate_genre_list(void) {
+static const char* genres[] = {
+  "Ambient",
+  "Blues",
+  "Classical",
+  "Country",
+  "Dance",
+  "Electronica",
+  "Folk",
+  "Jazz",
+  "Latin",
+  "Pop",
+  "Rap",
+  "Reggae",
+  "Rock",
+  "Soul",
+  "Spoken Word",
+  NULL
+};
+
+static GtkTreeModel* create_genre_list(void) {
   GtkListStore *store;
-  const GenreMap *p;
-  GList *genres;
-  GList *g;
+  const char **g = genres;
 
-  store = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);
-  p = genremap;
-  
-  genres = NULL;
-  while (p->genre != LAST_GENRE) {
-    if (p->genre == p->canonical) {
-      genres = g_list_insert_sorted (genres, (gpointer)p, genre_cmp);
-    }
-    ++p;
-  }
+  store = gtk_list_store_new (1, G_TYPE_STRING);
 
-  g = genres;
-  while (g != NULL) {
+  while (*g != NULL) {
     GtkTreeIter iter;
-    p = (GenreMap *)g->data;
     gtk_list_store_append (store, &iter);
-    gtk_list_store_set (store, &iter, 0, p->canonical, 1, sj_genre_name (p->canonical), -1);
-    g = g->next;
+    gtk_list_store_set (store, &iter, 0, *g++, -1);
   }
-  g_list_free (genres);
 
   return GTK_TREE_MODEL (store);
 }
 
+GtkWidget *
+sj_make_volume_button (void)
+{
+  GtkWidget *w = sj_volume_button_new (0.0, 1.0, 0.02);
+  sj_volume_button_set_value (SJ_VOLUME_BUTTON (w), 1.0);
+  return w;
+}
+
 static void
 upgrade_gconf (void)
 {
@@ -977,6 +1084,7 @@ int main (int argc, char **argv)
   struct poptOption options[] = {
     { NULL, '\0', POPT_ARG_INCLUDE_TABLE, NULL, 0, "GStreamer", NULL },
     { "auto-start", 'a', POPT_ARG_NONE, &autostart, 0, "Start extracting immediately", NULL},
+    { "play", 'p', POPT_ARG_NONE, &autoplay, 0, "Start playing immediately", NULL},
     POPT_TABLEEND
   };
 
@@ -993,6 +1101,8 @@ int main (int argc, char **argv)
                       NULL);
   g_set_application_name (_("Sound Juicer"));
 
+  sj_stock_init ();
+
   connection = bacon_message_connection_new ("sound-juicer");
   if (bacon_message_connection_get_is_server (connection) == FALSE) {
 	  bacon_message_connection_send (connection, RAISE_WINDOW);
@@ -1054,28 +1164,31 @@ int main (int argc, char **argv)
 
   select_all_menuitem = glade_xml_get_widget (glade, "select_all");
   deselect_all_menuitem = glade_xml_get_widget (glade, "deselect_all");
-  extract_menuitem = glade_xml_get_widget (glade, "extract_menuitem");
   title_entry = glade_xml_get_widget (glade, "title_entry");
   artist_entry = glade_xml_get_widget (glade, "artist_entry");
   duration_label = glade_xml_get_widget (glade, "duration_label");
-  genre_combo = glade_xml_get_widget (glade, "genre_combo");
+  genre_entry = glade_xml_get_widget (glade, "genre_entry");
   track_listview = glade_xml_get_widget (glade, "track_listview");
   extract_button = glade_xml_get_widget (glade, "extract_button");
+  extract_menuitem = glade_xml_get_widget (glade, "extract_menuitem");
+  play_button = glade_xml_get_widget (glade, "play_button");
+  play_menuitem = glade_xml_get_widget (glade, "play_menuitem");
   status_bar = glade_xml_get_widget (glade, "status_bar");
 
-  gtk_combo_box_set_model (GTK_COMBO_BOX (genre_combo), populate_genre_list ());
   {
-    GtkCellRenderer *renderer;
-    renderer = gtk_cell_renderer_text_new ();
-    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT(genre_combo), renderer, TRUE);
-    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(genre_combo), renderer, "text", 1);
+    GtkEntryCompletion *completion;
+    completion = gtk_entry_completion_new ();
+    gtk_entry_completion_set_model (completion, create_genre_list ());
+    gtk_entry_completion_set_text_column (completion, 0);
+    gtk_entry_completion_set_inline_completion (completion, TRUE);
+    gtk_entry_set_completion (GTK_ENTRY (genre_entry), completion);
   }
 
-  track_store = gtk_list_store_new (COLUMN_TOTAL, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_POINTER);
+  track_store = gtk_list_store_new (COLUMN_TOTAL, G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_POINTER);
   gtk_tree_view_set_model (GTK_TREE_VIEW (track_listview), GTK_TREE_MODEL (track_store));
   {
     GtkTreeViewColumn *column;
-    GtkCellRenderer *text_renderer, *toggle_renderer;
+    GtkCellRenderer *renderer;
     
     toggle_renderer = gtk_cell_renderer_toggle_new ();
     g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_extract_toggled), NULL);
@@ -1086,48 +1199,52 @@ int main (int argc, char **argv)
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
 
-    text_renderer = gtk_cell_renderer_text_new ();
-    column = gtk_tree_view_column_new_with_attributes (_("Track"),
-                                                       text_renderer,
-                                                       "text", COLUMN_NUMBER,
-                                                       NULL);
-    gtk_tree_view_column_set_resizable (column, TRUE);
+    column = gtk_tree_view_column_new ();
+    gtk_tree_view_column_set_title (column, _("Track"));
+    gtk_tree_view_column_set_expand (column, FALSE);
+    gtk_tree_view_column_set_resizable (column, FALSE);
+    renderer = gtk_cell_renderer_text_new ();
+    gtk_tree_view_column_pack_start (column, renderer, FALSE);
+    gtk_tree_view_column_add_attribute (column, renderer, "text", COLUMN_NUMBER);
+    renderer = gtk_cell_renderer_pixbuf_new ();
+    g_object_set (renderer, "stock-size", GTK_ICON_SIZE_MENU, "xalign", 0.0, NULL);
+    gtk_tree_view_column_pack_start (column, renderer, TRUE);
+    gtk_tree_view_column_set_cell_data_func (column, renderer, number_cell_icon_data_cb, NULL, NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
 
-    /* TODO: Do I need to create these every time or will a single one do? */
-    text_renderer = gtk_cell_renderer_text_new ();
-    g_signal_connect (text_renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_TITLE));
-    g_object_set (G_OBJECT (text_renderer), "editable", TRUE, NULL);
+    title_renderer = gtk_cell_renderer_text_new ();
+    g_signal_connect (title_renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_TITLE));
+    g_object_set (G_OBJECT (title_renderer), "editable", TRUE, NULL);
     column = gtk_tree_view_column_new_with_attributes (_("Title"),
-                                                       text_renderer,
+                                                       title_renderer,
                                                        "text", COLUMN_TITLE,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_column_set_expand (column, TRUE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
 
-    text_renderer = gtk_cell_renderer_text_new ();
+    artist_renderer = gtk_cell_renderer_text_new ();
     column = gtk_tree_view_column_new_with_attributes (_("Artist"),
-                                                       text_renderer,
+                                                       artist_renderer,
                                                        "text", COLUMN_ARTIST,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_column_set_expand (column, TRUE);
-    g_signal_connect (text_renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_ARTIST));
-    g_object_set (G_OBJECT (text_renderer), "editable", TRUE, NULL);
+    g_signal_connect (artist_renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_ARTIST));
+    g_object_set (G_OBJECT (artist_renderer), "editable", TRUE, NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
     
-    text_renderer = gtk_cell_renderer_text_new ();
+    renderer = gtk_cell_renderer_text_new ();
     column = gtk_tree_view_column_new_with_attributes (_("Duration"),
-                                                       text_renderer,
+                                                       renderer,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
-    gtk_tree_view_column_set_cell_data_func (column, text_renderer, duration_cell_data_cb, NULL, NULL);
+    gtk_tree_view_column_set_cell_data_func (column, renderer, duration_cell_data_cb, NULL, NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
   }
 
   update_ui_for_album (NULL);
-  gtk_widget_show_all (main_window);
+  gtk_widget_show (main_window);
 
   extractor = SJ_EXTRACTOR (sj_extractor_new());
   error = sj_extractor_get_new_error (extractor);
